@@ -1,16 +1,31 @@
-from typing import List
+from typing import List, Optional, Literal
 
-from langchain_community.agent_toolkits.base import BaseToolkit
-from langchain_core.tools import BaseTool
+from langchain_core.tools import BaseTool, BaseToolkit
+from pydantic import create_model, BaseModel, Field
 
 from .test_plan_wrapper import TestPlanApiWrapper
 from ...base.tool import BaseAction
 
+
 name = "azure_devops_plans"
+name_alias = "ado_plans"
+
 
 class AzureDevOpsPlansToolkit(BaseToolkit):
     tools: List[BaseTool] = []
 
+    @staticmethod
+    def toolkit_config_schema() -> BaseModel:
+        selected_tools = {x['name']: x['args_schema'].schema() for x in TestPlanApiWrapper.model_construct().get_available_tools()}
+        return create_model(
+            name_alias,
+            organization_url=(str, Field(description="ADO organization url")),
+            limit=(Optional[int], Field(description="ADO plans limit used for limitation of the list with results", default=5)),
+            token=(str, Field(description="ADO token", json_schema_extra={'secret': True})),
+            selected_tools=(List[Literal[tuple(selected_tools)]], Field(default=[], json_schema_extra={'args_schemas': selected_tools})),
+            __config__={'json_schema_extra': {'metadata': {"label": "ADO plans", "icon_url": None}}}
+        )
+
     @classmethod
     def get_toolkit(cls, selected_tools: list[str] | None = None, **kwargs):
         from os import environ
