@@ -1,14 +1,30 @@
-from typing import List
-from langchain_community.agent_toolkits.base import BaseToolkit
+from typing import List, Literal
 from .ado_wrapper import AzureDevOpsApiWrapper  # Import the API wrapper for Azure DevOps
-from langchain_core.tools import BaseTool
+from langchain_core.tools import BaseTool, BaseToolkit
+from pydantic import create_model, BaseModel, Field
+
 from ...base.tool import BaseAction
 
+
 name = "azure_devops_wiki"
+name_alias = 'ado_wiki'
+
 
 class AzureDevOpsWikiToolkit(BaseToolkit):
     tools: List[BaseTool] = []
 
+    @staticmethod
+    def toolkit_config_schema() -> BaseModel:
+        selected_tools = {x['name']: x['args_schema'].schema() for x in AzureDevOpsApiWrapper.model_construct().get_available_tools()}
+        return create_model(
+            name_alias,
+            organization_url=(str, Field(description="ADO organization url")),
+            project=(str, Field(description="ADO project")),
+            token=(str, Field(description="ADO token", json_schema_extra={'secret': True})),
+            selected_tools=(List[Literal[tuple(selected_tools)]], Field(default=[], json_schema_extra={'args_schemas': selected_tools})),
+            __config__={'json_schema_extra': {'metadata': {"label": "ADO wiki", "icon_url": None}}}
+        )
+
     @classmethod
     def get_toolkit(cls, selected_tools: list[str] | None = None, **kwargs):
         from os import environ
