@@ -1,14 +1,16 @@
 import json
 import logging
-from typing import Optional, Any
+from typing import Optional, Any, Dict
 
 from azure.devops.connection import Connection
+from azure.devops.released.work_item_tracking import TeamContext
 from azure.devops.released.work_item_tracking import Wiql
 from azure.devops.released.work_item_tracking import WorkItemTrackingClient
-from langchain_core.pydantic_v1 import root_validator, BaseModel
+from pydantic import model_validator, BaseModel
 from langchain_core.tools import ToolException
 from msrest.authentication import BasicAuthentication
-from pydantic import create_model, Field, PrivateAttr
+from pydantic import create_model, PrivateAttr
+from pydantic.fields import Field
 
 logger = logging.getLogger(__name__)
 
@@ -25,13 +27,20 @@ create_wi_field = """JSON of the work item fields to create in Azure DevOps, i.e
 ADOWorkItemsSearch = create_model(
     "AzureDevOpsSearchModel",
     query=(str, Field(description="WIQL query for searching Azure DevOps work items")),
+    limit=(Optional[int], Field(description="Number of items to return. IMPORTANT: Tool returns all items if limit=-1. If parameter is not provided then the value will be taken from tool configuration.", default=None)),
     fields=(Optional[list[str]], Field(description="Comma-separated list of requested fields", default=None))
 )
 
 ADOCreateWorkItem = create_model(
     "AzureDevOpsCreateWorkItemModel",
     work_item_json=(str, Field(description=create_wi_field)),
-    wi_type=(str, Field(description="Work item type, e.g. 'Task', 'Issue' or  'EPIC'", default="Task"))
+    wi_type=(Optional[str], Field(description="Work item type, e.g. 'Task', 'Issue' or  'EPIC'", default="Task"))
+)
+
+ADOUpdateWorkItem = create_model(
+    "AzureDevOpsUpdateWorkItemModel",
+    id=(str, Field(description="ID of work item required to be updated")),
+    work_item_json=(str, Field(description=create_wi_field))
 )
 
 ADOGetWorkItem = create_model(
@@ -42,18 +51,32 @@ ADOGetWorkItem = create_model(
     expand=(Optional[str], Field(description="The expand parameters for work item attributes. Possible options are { None, Relations, Fields, Links, All }.", default=None))
 )
 
+ADOLinkWorkItem = create_model(
+    "ADOLinkWorkItem",
+    source_id=(int, Field(description="ID of the work item you plan to add link to", default=None)),
+    target_id=(int, Field(description="ID of the work item linked to source one", default=None)),
+    link_type=(str, Field(description="Link type: System.LinkTypes.Dependency-forward, etc.", default=None)),
+    attributes=(Optional[dict], Field(description="Dict with attributes used for work items linking. Example: `comment`, etc. and syntax 'comment': 'Some linking comment'", default=None))
+)
+
+ADOGetLinkType = create_model(
+    "ADOGetLinkType",
+)
+
 
 class AzureDevOpsApiWrapper(BaseModel):
     organization_url: str
     project: str
     token: str
     limit: Optional[int] = 5
-    _client: Optional[WorkItemTrackingClient] = PrivateAttr()  # Private attribute for the work item tracking client
+    _client: Optional[WorkItemTrackingClient] = PrivateAttr()
+    _relation_types: Dict = PrivateAttr(default_factory=dict) # track actual relation types for instance
 
     class Config:
         arbitrary_types_allowed = True  # Allow arbitrary types (e.g., WorkItemTrackingClient)
 
-    @root_validator(pre=True)
+    @model_validator(mode='before')
+    @classmethod
     def validate_toolkit(cls, values):
         """Validate and set up the Azure DevOps client."""
         try:
@@ -97,32 +120,35 @@ class AzureDevOpsApiWrapper(BaseModel):
 
         return parsed_items
 
-    def create_work_item(self, work_item_json: str, wi_type="Task"):
-        """Create a work item in Azure DevOps."""
+    def _transform_work_item(self, work_item_json: str):
         try:
             # Convert the input JSON to a Python dictionary
             params = json.loads(work_item_json)
         except (json.JSONDecodeError, ValueError) as e:
-            return ToolException(f"Issues during attempt to parse work_item_json: {e}")
+            raise ToolException(f"Issues during attempt to parse work_item_json: {e}")
 
-        try:
-            if 'fields' not in params:
-                raise ToolException("The 'fields' property is missing from the work_item_json.")
+        if 'fields' not in params:
+            raise ToolException("The 'fields' property is missing from the work_item_json.")
 
             # Transform the dictionary into a list of JsonPatchOperation objects
-            patch_document = [
-                {
-                    "op": "add",
-                    "path": f"/fields/{field}",
-                    "value": value
-                }
-                for field, value in params["fields"].items()
-            ]
+        patch_document = [
+            {
+                "op": "add",
+                "path": f"/fields/{field}",
+                "value": value
+            }
+            for field, value in params["fields"].items()
+        ]
+        return patch_document
 
-            # Validate that the Azure DevOps client is initialized
-            if not self._client:
-                return ToolException("Azure DevOps client not initialized.")
+    def create_work_item(self, work_item_json: str, wi_type="Task"):
+        """Create a work item in Azure DevOps."""
+        try:
+            patch_document = self._transform_work_item(work_item_json)
+        except Exception as e:
+            return ToolException(f"Issues during attempt to parse work_item_json: {str(e)}")
 
+        try:
             # Use the transformed patch_document to create the work item
             work_item = self._client.create_work_item(
                 document=patch_document,
@@ -133,11 +159,61 @@ class AzureDevOpsApiWrapper(BaseModel):
         except Exception as e:
             if "unknown value" in str(e):
                 logger.error(f"Unable to create work item due to incorrect assignee: {e}")
-                raise ToolException(f"Unable to create work item due to incorrect assignee: {e}")
+                return ToolException(f"Unable to create work item due to incorrect assignee: {e}")
             logger.error(f"Error creating work item: {e}")
             return ToolException(f"Error creating work item: {e}")
 
-    def search_work_items(self, query: str, fields=None):
+    def update_work_item(self, id: str, work_item_json: str):
+        """Updates existing work item per defined data"""
+
+        try:
+            patch_document = self._transform_work_item(work_item_json)
+            work_item = self._client.update_work_item(id=id, document=patch_document, project=self.project)
+        except Exception as e:
+            return ToolException(f"Issues during attempt to parse work_item_json: {str(e)}")
+        return f"Work item ({work_item.id}) was updated."
+
+    def get_relation_types(self) -> dict:
+        """Returns dict of possible relation types per syntax: 'relation name': 'relation reference name'.
+        NOTE: reference name is used for adding links to the work item"""
+
+        if not self._relation_types:
+            # have to be called only once for session
+            relations = self._client.get_relation_types()
+            for relation in relations:
+                self._relation_types.update({relation.name: relation.reference_name})
+        return self._relation_types
+
+    def link_work_items(self, source_id, target_id, link_type, attributes: dict = None):
+        """Add the relation to the source work item with an appropriate attributes if any. User may pass attributes like name, etc."""
+
+        if not self._relation_types:
+            # check cached relation types and trigger its collection if it is empty by that moment
+            self.get_relation_types()
+        if link_type not in self._relation_types.values():
+            return ToolException(f"Link type is incorrect. You have to use proper relation's reference name NOT relation's name: {self._relation_types}")
+
+        relation = {
+            "rel": link_type,
+            "url": f"{self.organization_url}/_apis/wit/workItems/{target_id}"
+        }
+
+        if attributes:
+            relation.update({"attributes": attributes})
+
+        self._client.update_work_item(
+            document=[
+                {
+                    "op": "add",
+                    "path": "/relations/-",
+                    "value": relation
+                }
+            ],
+            id=source_id
+        )
+        return f"Work item {source_id} linked to {target_id} with link type {link_type}"
+
+    def search_work_items(self, query: str, limit: int = None, fields=None):
         """Search for work items using a WIQL query and dynamically fetch fields based on the query."""
         try:
             # Create a Wiql object with the query
@@ -148,7 +224,9 @@ class AzureDevOpsApiWrapper(BaseModel):
                 raise ToolException("Azure DevOps client not initialized.")
             logger.info(f"Search for work items using {query}")
             # Execute the WIQL query
-            work_items = self._client.query_by_wiql(wiql, top=self.limit, ).work_items
+            if not limit:
+                limit = self.limit
+            work_items = self._client.query_by_wiql(wiql, top=None if limit < 0 else limit, team_context=TeamContext(project=self.project)).work_items
 
             if not work_items:
                 return "No work items found."
@@ -187,6 +265,13 @@ class AzureDevOpsApiWrapper(BaseModel):
             else:
                 parsed_item.update(fields_data)
 
+            # extract relations if any
+            relations_data = work_item.relations
+            if relations_data:
+                parsed_item['relations'] = []
+                for relation in relations_data:
+                    parsed_item['relations'].append(relation.as_dict())
+
             return parsed_item
         except Exception as e:
             logger.error(f"Error getting work item: {e}")
@@ -208,11 +293,29 @@ class AzureDevOpsApiWrapper(BaseModel):
                 "args_schema": ADOCreateWorkItem,
                 "ref": self.create_work_item,
             },
+            {
+                "name": "update_work_item",
+                "description": self.update_work_item.__doc__,
+                "args_schema": ADOUpdateWorkItem,
+                "ref": self.update_work_item,
+            },
             {
                 "name": "get_work_item",
                 "description": self.get_work_item.__doc__,
                 "args_schema": ADOGetWorkItem,
                 "ref": self.get_work_item,
+            },
+            {
+                "name": "link_work_items",
+                "description": self.link_work_items.__doc__,
+                "args_schema": ADOLinkWorkItem,
+                "ref": self.link_work_items,
+            },
+            {
+                "name": "get_relation_types",
+                "description": self.get_relation_types.__doc__,
+                "args_schema": ADOGetLinkType,
+                "ref": self.get_relation_types,
             }
         ]
 
