@@ -1,11 +1,16 @@
+import json
 import logging
+import re
+import traceback
+from json import JSONDecodeError
+from typing import List, Optional, Any, Dict
+
 import requests
-from typing import List, Optional, Any
-from pydantic import create_model
-from pydantic.fields import FieldInfo
-from langchain_core.documents import Document
-from langchain_core.pydantic_v1 import root_validator, BaseModel
 from langchain_community.document_loaders.confluence import ContentFormat
+from langchain_core.documents import Document
+from langchain_core.tools import ToolException
+from markdownify import markdownify
+from pydantic import create_model, BaseModel, Field, model_validator
 from tenacity import (
     before_sleep_log,
     retry,
@@ -13,96 +18,115 @@ from tenacity import (
     wait_exponential,
 )
 
-
 logger = logging.getLogger(__name__)
 
 createPage = create_model(
     "createPage",
-    space=(str, FieldInfo(description="Confluence space that is used for page's creation", default=None)),
-    title=(str, FieldInfo(description="Title of the page")),
-    body=(str, FieldInfo(description="Body of the page")),
-    status=(str, FieldInfo(description="Page publishing option: 'current' for publish page, 'draft' to create draft.", default='current')),
-    parent_id=(str, FieldInfo(description="Page parent id (optional)", default=None)),
-    representation=(str, FieldInfo(description="Content representation format: storage for html, wiki for markdown", default='storage')),
-    label=(str, FieldInfo(description="Page label (optional)", default=None)),
+    space=(Optional[str], Field(description="Confluence space that is used for page's creation", default=None)),
+    title=(str, Field(description="Title of the page")),
+    body=(str, Field(description="Body of the page")),
+    status=(Optional[str], Field(description="Page publishing option: 'current' for publish page, 'draft' to create draft.", default='current')),
+    parent_id=(Optional[str], Field(description="Page parent id (optional)", default=None)),
+    representation=(Optional[str], Field(description="Content representation format: storage for html, wiki for markdown", default='storage')),
+    label=(Optional[str], Field(description="Page label (optional)", default=None)),
 )
 
 createPages = create_model(
     "createPages",
-    space=(str, FieldInfo(description="Confluence space that is used for pages creation", default=None)),
-    pages_info=(dict, FieldInfo(description="Content in key-value format: Title=Body")),
-    parent_id=(str, FieldInfo(description="Page parent id (optional)", default=None)),
-    status=(str, FieldInfo(description="Page publishing option: 'current' for publish page, 'draft' to create draft.", default='current')),
+    space=(Optional[str], Field(description="Confluence space that is used for pages creation", default=None)),
+    pages_info=(str, Field(description="""JSON string containing information about page name and its content per syntax: [{"page1_name": "page1_content"}, {"page2_name": "page2_content"}]""")),
+    parent_id=(Optional[str], Field(description="Page parent id (optional)", default=None)),
+    status=(Optional[str], Field(description="Page publishing option: 'current' for publish page, 'draft' to create draft.", default='current')),
 )
 
 deletePage = create_model(
     "deletePage",
-    page_id=(str, FieldInfo(description="Page id", default=None)),
-    page_title=(str, FieldInfo(description="Page title", default=None)),
+    page_id=(Optional[str], Field(description="Page id", default=None)),
+    page_title=(Optional[str], Field(description="Page title", default=None)),
 )
 
 updatePageById = create_model(
     "updatePageById",
-    page_id=(str, FieldInfo(description="Page id")),
-    representation=(str, FieldInfo(description="Content representation format: storage for html, wiki for markdown", default='storage')),
-    new_title=(str, FieldInfo(description="New page title", default=None)),
-    new_body=(str, FieldInfo(description="New page content", default=None)),
-    new_labels=(list, FieldInfo(description="Page labels", default=None)),
+    page_id=(str, Field(description="Page id")),
+    representation=(Optional[str], Field(description="Content representation format: storage for html, wiki for markdown", default='storage')),
+    new_title=(Optional[str], Field(description="New page title", default=None)),
+    new_body=(Optional[str], Field(description="New page content", default=None)),
+    new_labels=(Optional[list], Field(description="Page labels", default=None)),
 )
 
 updatePageByTitle = create_model(
     "updatePageByTitle",
-    page_title=(str, FieldInfo(description="Page title")),
-    representation=(str, FieldInfo(description="Content representation format: storage for html, wiki for markdown", default='storage')),
-    new_title=(str, FieldInfo(description="New page title", default=None)),
-    new_body=(str, FieldInfo(description="New page content", default=None)),
-    new_labels=(list, FieldInfo(description="Page labels", default=None)),
+    page_title=(str, Field(description="Page title")),
+    representation=(Optional[str], Field(description="Content representation format: storage for html, wiki for markdown", default='storage')),
+    new_title=(Optional[str], Field(description="New page title", default=None)),
+    new_body=(Optional[str], Field(description="New page content", default=None)),
+    new_labels=(Optional[list], Field(description="Page labels", default=None)),
 )
 
 updatePages = create_model(
     "updatePages",
-    page_ids=(list, FieldInfo(description="List of ids of pages to be updated", default=None)),
-    new_contents=(list, FieldInfo(description="List of new contents for each page. If content the same for all the pages then it should be a list with a single entry", default=None)),
-    new_labels=(list, FieldInfo(description="Page labels", default=None)),
+    page_ids=(Optional[list], Field(description="List of ids of pages to be updated", default=None)),
+    new_contents=(Optional[list], Field(description="List of new contents for each page. If content the same for all the pages then it should be a list with a single entry", default=None)),
+    new_labels=(Optional[list], Field(description="Page labels", default=None)),
 )
 
 updateLabels = create_model(
     "updateLabels",
-    page_ids=(list, FieldInfo(description="List of ids of pages to be updated", default=None)),
-    new_labels=(list, FieldInfo(description="Page labels", default=None)),
+    page_ids=(Optional[list], Field(description="List of ids of pages to be updated", default=None)),
+    new_labels=(Optional[list], Field(description="Page labels", default=None)),
 )
 
 getPageTree = create_model(
     "getPageTree",
-    page_id=(str, FieldInfo(description="Page id")),
+    page_id=(str, Field(description="Page id")),
 )
 
 pageExists = create_model(
     "pageExists",
-    title=(str, FieldInfo(description="Title of the page")),
+    title=(str, Field(description="Title of the page")),
 )
 
 getPagesWithLabel = create_model(
     "getPagesWithLabel",
-    label=(str, FieldInfo(description="Label of the pages")),
+    label=(str, Field(description="Label of the pages")),
 )
 
 searchPages = create_model(
     "searchPages",
-    query=(str, FieldInfo(description="Query text to search pages")),
+    query=(str, Field(description="Query text to search pages")),
+    skip_images=(Optional[bool], Field(description="Whether we need to skip existing images or not", default=False))
 )
 
 siteSearch = create_model(
     "siteSearch",
-    query=(str, FieldInfo(description="Query text to execute site search in Confluence")),
+    query=(str, Field(description="Query text to execute site search in Confluence")),
 )
 
 pageId = create_model(
     "pageId",
-    page_id=(str, FieldInfo(description="Id of page to be read")),
+    page_id=(str, Field(description="Id of page to be read")),
+    skip_images=(Optional[bool], Field(description="Whether we need to skip existing images or not", default=False)),
 )
 
+сonfluenceInput = create_model(
+     "сonfluenceInput",
+     method=(str, Field(description="The HTTP method to use for the request (GET, POST, PUT, DELETE, etc.). Required parameter.")),
+     relative_url=(str, Field(description="Required parameter: The relative URI for Confluence API. URI must start with a forward slash and '/rest/...'. Do not include query parameters in the URL, they must be provided separately in 'params'.")),
+     params=(Optional[str], Field(default="", description="Optional JSON of parameters to be sent in request body or query params. MUST be string with valid JSON. For search/read operations, you MUST always get minimum fields and set max results, until users ask explicitly for more fields. For search/read operations you must generate CQL query string and pass it as params."))
+ )
+
+
+def parse_payload_params(params: Optional[str]) -> Dict[str, Any]:
+    if params:
+        try:
+            return json.loads(params)
+        except JSONDecodeError:
+            stacktrace = traceback.format_exc()
+            return ToolException(f"Confluence tool exception. Passed params are not valid JSON. {stacktrace}")
+    return {}
+
 class ConfluenceAPIWrapper(BaseModel):
+    client: Any #: :meta private:
     base_url: str
     api_key: Optional[str] = None,
     username: Optional[str] = None
@@ -122,7 +146,8 @@ class ConfluenceAPIWrapper(BaseModel):
     ocr_languages: Optional[str] = None
     keep_newlines: Optional[bool] = True
 
-    @root_validator()
+    @model_validator(mode='before')
+    @classmethod
     def validate_toolkit(cls, values):
         try:
             from atlassian import Confluence  # noqa: F401
@@ -194,16 +219,17 @@ class ConfluenceAPIWrapper(BaseModel):
 
         return f"The page '{title}' was created under the parent page '{parent_id_filled}': '{page_details['link']}'. \nDetails: {str(page_details)}"
 
-    def create_pages(self, pages_info: dict, status: str = 'current', space: str = None, parent_id: str = None):
+    def create_pages(self, pages_info: str, status: str = 'current', space: str = None, parent_id: str = None):
         """ Creates a batch of pages in the Confluence space."""
         created_pages = []
         user_space = space if space else self.space
         logger.info(f"Pages will be created within the space {user_space}")
         # duplicate action to avoid extra api calls in downstream function
         parent_id_filled = parent_id if parent_id else self.client.get_space(user_space)['homepage']['id']
-        for title, body in pages_info.items():
-            created_page = self.create_page(title=title, body=body, status=status, parent_id=parent_id_filled, space=user_space)
-            created_pages.append(created_page)
+        for page_item in json.loads(pages_info):
+            for title, body in page_item.items():
+                created_page = self.create_page(title=title, body=body, status=status, parent_id=parent_id_filled, space=user_space)
+                created_pages.append(created_page)
         return str(created_pages)
 
     # delete after https://github.com/atlassian-api/atlassian-python-api/pull/1452 will be merged
@@ -351,12 +377,22 @@ class ConfluenceAPIWrapper(BaseModel):
 
     def get_pages_with_label(self, label: str):
         """ Gets pages with specific label in the Confluence space."""
+        return str(self._get_labeled_page(label))
+
+    def list_pages_with_label(self, label: str):
+        """ Lists the pages with specific label in the Confluence space."""
+        return [{'id': page['page_id'], 'title': page['page_title']} for page in self._get_labeled_page(label)]
+
+    def _get_labeled_page(self, label: str):
+        """Gets pages with specific label in the Confluence space."""
+
         start = 0
         pages_info = []
-        for _ in range(self.max_pages // self.limit):
+        for _ in range((self.max_pages + self.limit - 1) // self.limit):
             pages = self.client.get_all_pages_by_label(label, start=start, limit=self.limit) #, expand="body.view.value"
             if not pages:
                 break
+
             pages_info += [{
                 'page_id': page.metadata['id'],
                 'page_title': page.metadata['title'],
@@ -364,19 +400,19 @@ class ConfluenceAPIWrapper(BaseModel):
                 'content': page.page_content
             } for page in self.get_pages_by_id([page["id"] for page in pages])]
             start += self.limit
-        return str(pages_info)
+        return pages_info
 
     def is_public_page(self, page: dict) -> bool:
         """Check if a page is publicly accessible."""
         restrictions = self.client.get_all_restrictions_for_content(page["id"])
 
         return (
-            page["status"] == "current"
-            and not restrictions["read"]["restrictions"]["user"]["results"]
-            and not restrictions["read"]["restrictions"]["group"]["results"]
+                page["status"] == "current"
+                and not restrictions["read"]["restrictions"]["user"]["results"]
+                and not restrictions["read"]["restrictions"]["group"]["results"]
         )
 
-    def get_pages_by_id(self, page_ids: List[str]):
+    def get_pages_by_id(self, page_ids: List[str], skip_images: bool = False):
         """ Gets pages by id in the Confluence space."""
         for page_id in page_ids:
             get_page = retry(
@@ -396,22 +432,30 @@ class ConfluenceAPIWrapper(BaseModel):
             )
             if not self.include_restricted_content and not self.is_public_page(page):
                 continue
-            yield self.process_page(page)
-    
-    def read_page_by_id(self, page_id: str):
+            yield self.process_page(page, skip_images)
+
+    def read_page_by_id(self, page_id: str, skip_images: bool = False):
         """Reads a page by its id in the Confluence space."""
-        result = list(self.get_pages_by_id([page_id]))
-        return result[0].page_content if result else "Page not found"
-    
-    def _process_search(self, cql):
+
+        result = list(self.get_pages_by_id([page_id], skip_images))
+        if not result:
+            "Page not found"
+        return result[0].page_content
+        # return self._strip_base64_images(result[0].page_content) if skip_images else result[0].page_content
+
+    def _strip_base64_images(self, content):
+        base64_md_pattern = r'data:image/(png|jpeg|gif);base64,[a-zA-Z0-9+/=]+'
+        return re.sub(base64_md_pattern, "[Image Removed]", content)
+
+    def _process_search(self, cql, skip_images:bool = False):
         start = 0
         pages_info = []
-        for _ in range(self.max_pages // self.limit):
+        for _ in range((self.max_pages + self.limit - 1) // self.limit):
             pages = self.client.cql(cql, start=start, limit=self.limit).get("results", [])
             if not pages:
                 break
             page_ids = [page['content']['id'] for page in pages]
-            for page in self.get_pages_by_id(page_ids):
+            for page in self.get_pages_by_id(page_ids, skip_images):
                 page_info = {
                     'content': page.page_content,
                     'page_id': page.metadata['id'],
@@ -422,21 +466,21 @@ class ConfluenceAPIWrapper(BaseModel):
             start += self.limit
         return str(pages_info)
 
-    def search_pages(self, query: str):
+    def search_pages(self, query: str, skip_images: bool = False):
         """Search pages in Confluence by query text in title or page content."""
         if not self.space:
             cql = f'(type=page) and (title~"{query}" or text~"{query}")'
         else:
             cql = f'(type=page and space={self.__sanitize_confluence_space()}) and (title~"{query}" or text~"{query}")'
-        return self._process_search(cql)
+        return self._process_search(cql, skip_images)
 
-    def search_by_title(self, query: str):
+    def search_by_title(self, query: str, skip_images: bool = False):
         """Search pages in Confluence by query text in title."""
         if not self.space:
             cql = f'(type=page) and (title~"{query}")'
         else:
             cql = f'(type=page and space={self.__sanitize_confluence_space()}) and (title~"{query}")'
-        return self._process_search(cql)
+        return self._process_search(cql, skip_images)
 
     def site_search(self, query: str):
         """Search for pages in Confluence using site search by query text."""
@@ -459,7 +503,7 @@ class ConfluenceAPIWrapper(BaseModel):
             content.append(page_data)
         return '---'.join([str(page_data) for page_data in content])
 
-    def process_page(self, page: dict) -> Document:
+    def process_page(self, page: dict, skip_images: bool = False) -> Document:
         if self.keep_markdown_format:
             try:
                 from markdownify import markdownify
@@ -518,14 +562,14 @@ class ConfluenceAPIWrapper(BaseModel):
             metadata["when"] = page["version"]["when"]
 
         return Document(
-            page_content=text,
+            page_content=self._strip_base64_images(text) if skip_images else text,
             metadata=metadata,
         )
-    
+
     def process_attachment(
-        self,
-        page_id: str,
-        ocr_languages: Optional[str] = None,
+            self,
+            page_id: str,
+            ocr_languages: Optional[str] = None,
     ) -> List[str]:
         try:
             from PIL import Image  # noqa: F401
@@ -546,14 +590,14 @@ class ConfluenceAPIWrapper(BaseModel):
                 if media_type == "application/pdf":
                     text = title + self.process_pdf(absolute_url, ocr_languages)
                 elif (
-                    media_type == "image/png"
-                    or media_type == "image/jpg"
-                    or media_type == "image/jpeg"
+                        media_type == "image/png"
+                        or media_type == "image/jpg"
+                        or media_type == "image/jpeg"
                 ):
                     text = title + self.process_image(absolute_url, ocr_languages)
                 elif (
-                    media_type == "application/vnd.openxmlformats-officedocument"
-                    ".wordprocessingml.document"
+                        media_type == "application/vnd.openxmlformats-officedocument"
+                                      ".wordprocessingml.document"
                 ):
                     text = title + self.process_doc(absolute_url)
                 elif media_type == "application/vnd.ms-excel":
@@ -571,7 +615,37 @@ class ConfluenceAPIWrapper(BaseModel):
                     raise
 
         return texts
-    
+
+    def execute_generic_confluence(self, method: str, relative_url: str, params: Optional[str] = "") -> str:
+        """Generic Confluence Tool for Official Atlassian Confluence REST API to call, searching, creating, updating pages, etc."""
+        payload_params = parse_payload_params(params)
+        if method == "GET":
+            response = self.client.request(
+                method=method,
+                path=relative_url,
+                params=payload_params,
+                advanced_mode=True
+            )
+            response_text = self.process_search_response(relative_url, response)
+        else:
+            response = self.client.request(
+                method=method,
+                path=relative_url,
+                data=payload_params,
+                advanced_mode=True
+            )
+            response_text = response.text
+        response_string = f"HTTP: {method}{relative_url} -> {response.status_code}{response.reason}{response_text}"
+        logger.debug(response_string)
+        return response_string
+
+    def process_search_response(self, relative_url: str, response) -> str:
+        page_search_pattern = r'/rest/api/content/\d+'
+        if re.match(page_search_pattern, relative_url):
+            body = markdownify(response.text, heading_style="ATX")
+            return body
+        return response.text
+
     def get_available_tools(self):
         return [
             {
@@ -634,6 +708,12 @@ class ConfluenceAPIWrapper(BaseModel):
                 "description": self.get_pages_with_label.__doc__,
                 "args_schema": getPagesWithLabel,
             },
+            {
+                "name": "list_pages_with_label",
+                "ref": self.list_pages_with_label,
+                "description": self.list_pages_with_label.__doc__,
+                "args_schema": getPagesWithLabel,
+            },
             {
                 "name": "read_page_by_id",
                 "ref": self.read_page_by_id,
@@ -657,9 +737,15 @@ class ConfluenceAPIWrapper(BaseModel):
                 "ref": self.site_search,
                 "description": self.site_search.__doc__,
                 "args_schema": siteSearch,
+            },
+            {
+                "name": "execute_generic_confluence",
+                "description": self.execute_generic_confluence.__doc__,
+                "args_schema": сonfluenceInput,
+                "ref": self.execute_generic_confluence,
             }
         ]
-    
+
     def run(self, mode: str, *args: Any, **kwargs: Any):
         for tool in self.get_available_tools():
             if tool["name"] == mode:
