@@ -1,11 +1,16 @@
 import os
-from json import dumps
+import re
+from json import dumps, loads
+import fnmatch
 from typing import Dict, Any, Optional, List
 import tiktoken
-from pydantic import root_validator, create_model
-from pydantic.fields import FieldInfo
+from pydantic import model_validator, create_model, BaseModel, Field
 from langchain.utils import get_from_dict_or_env
 
+from logging import getLogger
+
+logger = getLogger(__name__)
+
 from langchain_community.tools.github.prompt import (
     COMMENT_ON_ISSUE_PROMPT,
     CREATE_BRANCH_PROMPT,
@@ -21,6 +26,7 @@ from langchain_community.tools.github.prompt import (
     OVERVIEW_EXISTING_FILES_IN_MAIN,
     READ_FILE_PROMPT,
     SET_ACTIVE_BRANCH_PROMPT,
+    SEARCH_ISSUES_AND_PRS_PROMPT,
 )
 
 from langchain_community.utilities.github import GitHubAPIWrapper
@@ -54,26 +60,80 @@ existing contents
 new contents
 >>>> NEW"""
 
+CREATE_ISSUE_PROMPT = """
+This tool allows you to create a new issue in a GitHub repository. **VERY IMPORTANT**: Your input to this tool MUST strictly follow these rules:
+
+- First, you must specify the title of the issue.
+
+Optionally you can specify:
+- a detailed description or body of the issue
+- labels for the issue, each separated by a comma. For labels, write `labels:` followed by a comma-separated list of labels.
+- assignees for the issue, each separated by a comma. For assignees, write `assignees:` followed by a comma-separated 
+list of GitHub usernames.
+
+Ensure that each command (`labels:` and `assignees:`) starts in a new line, if used.
+
+For example, if you would like to create an issue titled "Fix login bug" with a body explaining the problem and tagged with `bug` 
+and `urgent` labels, plus assigned to `user123`, you would pass in the following string:
+
+Fix login bug
+
+The login button isn't responding on the main page. Reproduced on various devices.
+
+labels: bug, urgent
+assignees: user123
+"""
+
+UPDATE_ISSUE_PROMPT = """
+This tool allows you to update an existing issue in a GitHub repository. **VERY IMPORTANT**: Your input to this tool MUST strictly follow 
+these rules:
+- You must specify the repository name where the issue exists.
+- You must specify the issue ID that you wish to update.
+
+Optional fields:
+- You can then provide the new title of the issue.
+- You can provide a new detailed description or body of the issue.
+- You can specify new labels for the issue, each separated by a comma.
+- You can specify new assignees for the issue, each separated by a comma.
+- You can change the state of the issue to either 'open' or 'closed'.
+
+If assignees or labels are not passed or passed as empty lists they will be removed from the issue.
+
+For example, to update an issue in the 'octocat/Hello-World' repository with ID 42, changing the title and closing the issue, the input should be:
+
+octocat/Hello-World
+
+42
+
+New Issue Title
+
+New detailed description goes here.
+
+labels: bug, ui
+
+assignees: user1, user2
+
+closed
+"""
 
 SearchCode = create_model(
     "SearchCodeModel",
-    query=(str, FieldInfo(description=("A keyword-focused natural language "
-                                       "search query for code, e.g. `MyFunctionName()`.")))
-    )
+    query=(str, Field(description=("A keyword-focused natural language "
+                                   "search query for code, e.g. `MyFunctionName()`.")))
+)
 
 GetIssue = create_model(
     "GetIssue",
-    issue_number=(str, FieldInfo(description="Issue number as an integer, e.g. `42`"))
+    issue_number=(str, Field(description="Issue number as a string, e.g. `42`"))
 )
 
 GetPR = create_model(
     "GetPR",
-    pr_number=(str, FieldInfo(description="The PR number as an integer, e.g. `12`"))
+    pr_number=(str, Field(description="The PR number as a string, e.g. `12`"))
 )
 DirectoryPath = create_model(
     "DirectoryPath",
-    directory_path=(str, FieldInfo(
-        default="",
+    directory_path=(str, Field(
         description=(
             "The path of the directory, e.g. `some_dir/inner_dir`."
             " Only input a string, do not include the parameter name."
@@ -87,7 +147,7 @@ NoInput = create_model(
 
 ReadFile = create_model(
     "ReadFile",
-    file_path=(str, FieldInfo(
+    file_path=(str, Field(
         description=(
             "The full file path of the file you would like to read where the "
             "path must NOT start with a slash, e.g. `some_dir/my_file.py`."
@@ -97,37 +157,37 @@ ReadFile = create_model(
 
 CreateBranchName = create_model(
     "CreateBranchName",
-    proposed_branch_name=(str, FieldInfo(
+    proposed_branch_name=(str, Field(
         description="The name of the branch, e.g. `my_branch`."
     ))
 )
 
 UpdateFile = create_model(
     "UpdateFile",
-    file_query=(str, FieldInfo(
+    file_query=(str, Field(
         description="Strictly follow the provided rules."
     ))
 )
 
 CreateFile = create_model(
     "CreateFile",
-    file_path=(str, FieldInfo(description="Path of a file to be created.")),
-    file_contents=(str, FieldInfo(description="Content of a file to be put into chat."))
+    file_path=(str, Field(description="Path of a file to be created.")),
+    file_contents=(str, Field(description="Content of a file to be put into chat."))
 )
 
 CreatePR = create_model(
     "CreatePR",
-    pr_query=(str, FieldInfo(description="Follow the required formatting."))
+    pr_query=(str, Field(description="Follow the required formatting."))
 )
 
 CommentOnIssue = create_model(
     "CommentOnIssue",
-    comment_query=(str, FieldInfo(default=..., description="Follow the required formatting."))
+    comment_query=(str, Field(description="Follow the required formatting."))
 )
 
 DeleteFile = create_model(
     "DeleteFile",
-    file_path=(str, FieldInfo(
+    file_path=(str, Field(
         description=(
             "The full file path of the file you would like to delete"
             " where the path must NOT start with a slash, e.g."
@@ -139,9 +199,112 @@ DeleteFile = create_model(
 
 BranchName = create_model(
     "BranchName",
-    branch_name=(str, FieldInfo(description="The name of the branch, e.g. `my_branch`."))
+    branch_name=(str, Field(description="The name of the branch, e.g. `my_branch`."))
 )
 
+SearchIssues = create_model(
+    "SearchIssues",
+    search_query=(
+        str,
+        Field(
+            description="Keywords or query for searching issues and PRs in Github (supports GitHub search syntax)"
+        ),
+    ),
+    repo_name=(
+        Optional[str],
+        Field(
+            description="Name of the repository to search issues in. If None, use the initialized repository.",
+            default=None
+        ),
+    ),
+    max_count=(
+        Optional[str],
+        Field(
+            description="Default is 30. This determines max size of returned list with issues",
+            default=30
+        ),
+    ),
+)
+
+CreateIssue = create_model(
+    "CreateIssue",
+    title=(str, Field(description="The title of the issue.")),
+    body=(
+        Optional[str],
+        Field(
+            description="The body or description of the issue providing details and context.",
+            default=None
+        ),
+    ),
+    labels=(
+        Optional[List[str]],
+        Field(
+            default=None,
+            description="A list of labels to apply to the issue. This should be a list of strings.",
+        ),
+    ),
+    assignees=(
+        Optional[List[str]],
+        Field(
+            default=None,
+            description="A list of GitHub usernames to whom the issue should be assigned. This should be a list of strings.",
+        ),
+    ),
+    repo_name=(
+        Optional[str],
+        Field(description="The name of the repository where the issue exists.",
+              default=None),
+    ),
+)
+
+UpdateIssue = create_model(
+    "UpdateIssue",
+    issue_id=(int, Field(description="The ID of the issue to be updated.")),
+    repo_name=(
+        Optional[str],
+        Field(description="The name of the repository where the issue exists.",
+              default=None),
+    ),
+    title=(
+        Optional[str],
+        Field(default=None, description="New title for the issue if updating."),
+    ),
+    body=(
+        Optional[str],
+        Field(
+            default=None,
+            description="New detailed description for the issue if updating.",
+        ),
+    ),
+    labels=(
+        Optional[List[str]],
+        Field(
+            default=None,
+            description="New list of labels to apply to the issue if updating.",
+        ),
+    ),
+    assignees=(
+        Optional[List[str]],
+        Field(
+            default=None,
+            description="New list of GitHub usernames to assign to the issue if updating.",
+        ),
+    ),
+    state=(
+        Optional[str],
+        Field(
+            default=None,
+            description="The new state of the issue ('open' or 'closed') if updating.",
+        ),
+    ),
+)
+
+LoaderSchema = create_model(
+    "LoaderSchema",
+    branch=(Optional[str], Field(description="The branch to set as active before listing files. If None, the current active branch is used.")),
+    whitelist=(Optional[List[str]], Field(description="A list of file extensions or paths to include. If None, all files are included.")),
+    blacklist=(Optional[List[str]], Field(description="A list of file extensions or paths to exclude. If None, no files are excluded."))
+)
 
 class AlitaGitHubAPIWrapper(GitHubAPIWrapper):
     github: Any  #: :meta private:
@@ -154,26 +317,26 @@ class AlitaGitHubAPIWrapper(GitHubAPIWrapper):
     github_password: Optional[str] = None
     github_app_id: Optional[str] = None
     github_app_private_key: Optional[str] = None
-    
-    
-    @root_validator(pre=True)
+
+    @model_validator(mode='before')
+    @classmethod
     def validate_environment(cls, values: Dict) -> Dict:
-         
-        github_app_id = get_from_dict_or_env(values, 
-                                             "github_app_id", 
+
+        github_app_id = get_from_dict_or_env(values,
+                                             "github_app_id",
                                              "GITHUB_APP_ID",
                                              default='')
-        
+
         github_app_private_key = get_from_dict_or_env(
-            values, 
-            "github_app_private_key", 
-            "GITHUB_APP_PRIVATE_KEY", 
+            values,
+            "github_app_private_key",
+            "GITHUB_APP_PRIVATE_KEY",
             default=''
         )
-        
+
         github_access_token = get_from_dict_or_env(
-            values, "github_access_token",  "GITHUB_ACCESS_TOKEN", default='')
-        
+            values, "github_access_token", "GITHUB_ACCESS_TOKEN", default='')
+
         github_username = get_from_dict_or_env(
             values, "github_username", "GITHUB_USERNAME", default='')
         github_password = get_from_dict_or_env(
@@ -187,12 +350,12 @@ class AlitaGitHubAPIWrapper(GitHubAPIWrapper):
         github_base_branch = get_from_dict_or_env(
             values, "github_base_branch", "GITHUB_BASE_BRANCH", default="main")
 
-        if github_app_private_key and os.path.exists(github_app_private_key):    
+        if github_app_private_key and os.path.exists(github_app_private_key):
             with open(github_app_private_key, "r") as f:
                 private_key = f.read()
         else:
             private_key = github_app_private_key
-        
+
         try:
             from github import Auth, GithubIntegration, Github
             from github.Consts import DEFAULT_BASE_URL
@@ -201,9 +364,9 @@ class AlitaGitHubAPIWrapper(GitHubAPIWrapper):
                 "PyGithub is not installed. "
                 "Please install it with `pip install PyGithub`"
             )
-            
+
         github_base_url = get_from_dict_or_env(
-            values, "github_base_url", "GITHUB_BASE_URL", default=DEFAULT_BASE_URL)        
+            values, "github_base_url", "GITHUB_BASE_URL", default=DEFAULT_BASE_URL)
         if github_access_token:
             print(github_access_token)
             auth = Auth.Token(github_access_token)
@@ -213,7 +376,7 @@ class AlitaGitHubAPIWrapper(GitHubAPIWrapper):
             auth = Auth.AppAuth(github_app_id, private_key)
         else:
             auth = None
-            
+
         if auth is None:
             g = Github(base_url=github_base_url)
         elif github_app_id and private_key:
@@ -229,13 +392,12 @@ class AlitaGitHubAPIWrapper(GitHubAPIWrapper):
         values["github_repository"] = github_repository
         values["active_branch"] = active_branch
         values["github_base_branch"] = github_base_branch
-        
+
         return values
-    
-    
-    def _get_files(self, directory_path: str, ref: str) -> List[str]:
+
+    def _get_files(self, directory_path: str, ref: str) -> List[str]:  
         from github import GithubException
-        
+
         files: List[str] = []
         try:
             contents = self.github_repo_instance.get_contents(
@@ -250,8 +412,8 @@ class AlitaGitHubAPIWrapper(GitHubAPIWrapper):
                 contents.extend(self.github_repo_instance.get_contents(file_content.path))
             else:
                 files.append(file_content)
-        return str(files)
-    
+        return [file.path for file in files]
+
     def get_files_from_directory(self, directory_path: str) -> str:
         """
         Recursively fetches files from a directory in the repo.
@@ -262,8 +424,8 @@ class AlitaGitHubAPIWrapper(GitHubAPIWrapper):
         Returns:
             str: List of file paths, or an error message.
         """
-        
-        return self._get_files(directory_path, self.active_branch)
+
+        return dumps(self._get_files(directory_path, self.active_branch))
 
     def get_issue(self, issue_number: str) -> str:
         """
@@ -273,7 +435,6 @@ class AlitaGitHubAPIWrapper(GitHubAPIWrapper):
             str: A dictionary containing information about the issue.
         """
         return dumps(super().get_issue(int(issue_number)))
-    
 
     def list_files_in_main_branch(self) -> str:
         """
@@ -282,7 +443,7 @@ class AlitaGitHubAPIWrapper(GitHubAPIWrapper):
         Returns:
             str: A plaintext report containing the paths and names of the files.
         """
-        return self._get_files("", self.github_base_branch)
+        return dumps(self._get_files("", self.github_base_branch))
 
     def list_files_in_bot_branch(self) -> str:
         """
@@ -291,13 +452,12 @@ class AlitaGitHubAPIWrapper(GitHubAPIWrapper):
         Returns:
             str: A plaintext report containing the paths and names of the files.
         """
-        return self._get_files("", self.active_branch)
-
+        return dumps(self._get_files("", self.active_branch))
 
     def get_pull_request(self, pr_number: str) -> str:
         """
         Fetches information about a specific pull request.
-        
+
         Returns:
             str: A dictionary containing information about the pull request.
         """
@@ -319,6 +479,7 @@ class AlitaGitHubAPIWrapper(GitHubAPIWrapper):
         add_to_dict(response_dict, "title", pull.title)
         add_to_dict(response_dict, "number", str(pr_number))
         add_to_dict(response_dict, "body", str(pull.body))
+        add_to_dict(response_dict, "pr_url", str(pull.html_url))
 
         comments: List[str] = []
         page = 0
@@ -354,7 +515,7 @@ class AlitaGitHubAPIWrapper(GitHubAPIWrapper):
     def list_pull_request_diffs(self, pr_number: str) -> str:
         """
         Fetches the files included in a pull request.
-        
+
         Returns:
             str: A list of files and pathes to then included in the pull request.
         """
@@ -371,7 +532,7 @@ class AlitaGitHubAPIWrapper(GitHubAPIWrapper):
                     "path": path,
                     "patch": patch
                 }
-            )    
+            )
         return dumps(data)
 
     def create_branch(self, proposed_branch_name: str) -> str:
@@ -402,7 +563,7 @@ class AlitaGitHubAPIWrapper(GitHubAPIWrapper):
                     "created successfully, and set as current active branch."
                 )
             except GithubException as e:
-                if e.status == 422 and "Reference already exists" in e.data["message"]:
+                if (e.status == 422 and "Reference already exists" in e.data["message"]):
                     i += 1
                     new_branch_name = f"{proposed_branch_name}_v{i}"
                 else:
@@ -457,22 +618,22 @@ class AlitaGitHubAPIWrapper(GitHubAPIWrapper):
             return "Created file " + file_path
         except Exception as e:
             return "Unable to make file due to error:\n" + str(e)
-    
+
     def extract_old_new_pairs(self, file_query):
         # Split the file content by lines
         code_lines = file_query.split("\n")
-        
+
         # Initialize lists to hold the contents of OLD and NEW sections
         old_contents = []
         new_contents = []
-        
+
         # Initialize variables to track whether the current line is within an OLD or NEW section
         in_old_section = False
         in_new_section = False
-        
+
         # Temporary storage for the current section's content
         current_section_content = []
-        
+
         # Iterate through each line in the file content
         for line in code_lines:
             # Check for OLD section start
@@ -480,36 +641,36 @@ class AlitaGitHubAPIWrapper(GitHubAPIWrapper):
                 in_old_section = True
                 current_section_content = []  # Reset current section content
                 continue  # Skip the line with the marker
-            
+
             # Check for OLD section end
             if ">>>> OLD" in line:
                 in_old_section = False
                 old_contents.append("\n".join(current_section_content).strip())  # Add the captured content
                 current_section_content = []  # Reset current section content
                 continue  # Skip the line with the marker
-            
+
             # Check for NEW section start
             if "NEW <<<" in line:
                 in_new_section = True
                 current_section_content = []  # Reset current section content
                 continue  # Skip the line with the marker
-            
+
             # Check for NEW section end
             if ">>>> NEW" in line:
                 in_new_section = False
                 new_contents.append("\n".join(current_section_content).strip())  # Add the captured content
                 current_section_content = []  # Reset current section content
                 continue  # Skip the line with the marker
-            
+
             # If currently in an OLD or NEW section, add the line to the current section content
             if in_old_section or in_new_section:
                 current_section_content.append(line)
-        
+
         # Pair the OLD and NEW contents
         paired_contents = list(zip(old_contents, new_contents))
-        
+
         return paired_contents
-    
+
     def update_file(self, file_query: str) -> str:
         """
         Updates a file with new content.
@@ -535,9 +696,9 @@ class AlitaGitHubAPIWrapper(GitHubAPIWrapper):
                 "Please create a new branch and try again."
             )
         try:
-            
+
             file_path: str = file_query.split("\n")[0]
-            
+
             file_content = self.read_file(file_path)
             updated_file_content = file_content
             for old, new in self.extract_old_new_pairs(file_query):
@@ -565,6 +726,209 @@ class AlitaGitHubAPIWrapper(GitHubAPIWrapper):
         except Exception as e:
             return "Unable to update file due to error:\n" + str(e)
 
+    def validate_search_query(self, query: str) -> bool:
+        """
+        Validates a search query against expected GitHub search syntax using regular expressions.
+
+        Parameters:
+            query (str): The search query to validate.
+
+        Returns:
+            bool: True if valid, False otherwise.
+        """
+        pattern = r"^(\w+:)?[\w\s]+$"
+        return bool(re.match(pattern, query))
+
+    def search_issues(self, search_query: str, repo_name: Optional[str] = None, max_count: int = 30) -> str:
+        """
+        Searches for issues in a specific repository or a default initialized repository
+        based on a search query using GitHub's search feature.
+
+        Parameters:
+            search_query (str): Keywords or query for searching issues and PRs in Github (supports GitHub search syntax).
+            repo_name (Optional[str]): Name of the repository to search issues in. If None, use the initialized repository.
+            max_count (int): Default is 30. This determines max size of returned list with issues
+
+        Returns:
+            str: JSON string containing a list of issues and PRs with their details (id, title, description, status, URL, type)
+        """
+        try:
+            if not self.validate_search_query(search_query):
+                return "Invalid search query. Please ensure it matches expected GitHub search syntax."
+        
+            target_repo = self.github_repo_instance.full_name if repo_name is None else repo_name
+
+            query = f"repo:{target_repo} {search_query}"
+            search_result = self.github.search_issues(query)
+
+            if not search_result.totalCount:
+                return "No issues or PRs found matching your query."
+
+            matching_issues = []
+            
+            count = min(max_count, search_result.totalCount)
+            for issue in search_result[:count]:
+                issue_details = {
+                    "id": issue.number,
+                    "title": issue.title,
+                    "description": issue.body,
+                    "status": issue.state,
+                    "url": issue.html_url,
+                    "type": "PR" if issue.pull_request else "Issue"
+                }
+                matching_issues.append(issue_details)
+
+            return dumps(matching_issues)
+        except Exception as e:
+            return "An error occurred while searching issues:\n" + str(e)
+        
+    def create_issue(self, title: str, body: Optional[str] = None, repo_name: Optional[str] = None, labels: Optional[List[str]] = None, assignees: Optional[List[str]] = None) -> str:
+        """
+        Creates a new issue in the GitHub repository.
+
+        Parameters:
+            title (str): The title of the issue.
+            body (Optional[str]): The detailed description of the issue.
+            labels (Optional[List[str]]): An optional list of labels to attach to the issue.
+            assignees (Optional[List[str]]): An optional list of GitHub usernames to assign the issue to.
+
+        Returns:
+            str: A success or failure message along with the URL to the newly created issue.
+        """
+        try:
+            repo = self.github.get_repo(repo_name) if repo_name else self.github_repo_instance
+
+            if not repo:
+                return "GitHub repository instance is not found or not initialized."
+            
+            issue = repo.create_issue(
+                title=title,
+                body=body,
+                labels=labels if labels else [],
+                assignees=assignees if assignees else []
+            )
+
+            return f"Issue created successfully! ID:{issue.number}, URL: {issue.html_url}"
+        except Exception as e:
+            return f"An error occurred while creating the issue: {str(e)}"
+        
+    def update_issue(self, issue_id: int, title: Optional[str] = None, 
+                 body: Optional[str] = None, labels: Optional[List[str]] = None, 
+                 assignees: Optional[List[str]] = None, state: Optional[str] = None,
+                 repo_name: Optional[str] = None) -> str:
+        """
+        Updates an existing issue in a specified GitHub repository.
+
+        Parameters:
+            issue_id (int): ID of the issue to update.
+            title (str): New title of the issue, if updating.
+            body (Optional[str]): New detailed description of the issue, if updating.
+            labels (Optional[List[str]]): New list of labels to apply to the issue, if updating.
+            assignees (Optional[List[str]]): New list of GitHub usernames to assign to the issue, if updating.
+            state (Optional[str]): New state of the issue ("open" or "closed"), if updating.
+            repo_name (Optional[str]): Name of the repository where the issue exists.
+
+        Returns:
+            str: A confirmation message including the updated issue details, or an error message.
+        """
+        if not issue_id:
+                return "Issue ID is required."
+        try:
+            repo = self.github.get_repo(repo_name) if repo_name else self.github_repo_instance
+            issue = repo.get_issue(number=issue_id)
+
+            if not issue:
+                return f"Issue with #{issue_id} has not been found."
+            
+            if labels is None or labels == []:
+                current_labels = [label.name for label in issue.get_labels()]
+                for label in current_labels:
+                    issue.remove_from_labels(label)
+
+            if not assignees:
+                for assignee in issue.assignees:
+                    issue.remove_from_assignees(assignee)
+
+            update_fields = {}
+            if title:
+                update_fields["title"] = title
+            if body:
+                update_fields["body"] = body
+            if labels:
+                update_fields["labels"] = labels
+            if assignees:
+                update_fields["assignees"] = assignees
+            if state:
+                update_fields["state"] = state
+
+            issue.edit(**update_fields)
+
+            return f"Issue updated successfully! Updated details: ID: {issue.number}, URL: {issue.html_url}"
+        except Exception as e:
+            return f"An error occurred while updating the issue: {str(e)}"
+    
+    def _read_file(self, file_path: str, branch: str) -> str:
+        """
+        Read a file from specified branch
+        Parameters:
+            file_path(str): the file path
+            branch(str): the branch to read the file from
+        Returns:
+            str: The file decoded as a string, or an error message if not found
+        """
+        try:
+            file = self.github_repo_instance.get_contents(file_path, ref=branch)
+            return file.decoded_content.decode("utf-8")
+        except Exception as e:
+            from traceback import format_exc
+            logger.info(format_exc())
+            return f"File not found `{file_path}` on branch `{branch}`. Error: {str(e)}"
+
+    
+    def loader(self,
+               branch: Optional[str] = None,
+               whitelist: Optional[List[str]] = None, 
+               blacklist: Optional[List[str]] = None) -> str:
+        """
+        Generates file content from the specified branch while honoring whitelist and blacklist patterns.
+
+        Parameters:
+            branch (Optional[str]): The branch to set as active before listing files. If None, the current active branch is used.
+            whitelist (Optional[List[str]]): A list of file extensions or paths to include. If None, all files are included.
+            blacklist (Optional[List[str]]): A list of file extensions or paths to exclude. If None, no files are excluded.
+
+        Returns:
+            generator: A generator that yields the content of files that match the whitelist and do not match the blacklist.
+
+        Example:
+            # Set the active branch to 'feature-branch' and include only '.py' files, excluding 'test_' files
+            file_generator = loader(branch='feature-branch', whitelist=['*.py'], blacklist=['*test_*'])
+            for file_content in file_generator:
+                print(file_content)
+
+        Notes:
+            - The whitelist and blacklist patterns use Unix shell-style wildcards.
+            - If both whitelist and blacklist are provided, a file must match the whitelist and not match the blacklist to be included.
+        """
+        from ..code.loaders.codeparser import parse_code_files_for_db
+        _files = self._get_files("", branch or self.active_branch)
+        logger.info(f"Files in branch: {self.active_branch}")
+        def is_whitelisted(file_path: str) -> bool:
+            if whitelist:
+                return any(fnmatch.fnmatch(file_path, pattern) for pattern in whitelist)
+            return True
+
+        def is_blacklisted(file_path: str) -> bool:
+            if blacklist:
+                return any(fnmatch.fnmatch(file_path, pattern) for pattern in blacklist)
+            return False
+        
+        def file_content_generator():
+            for file in _files:
+                if is_whitelisted(file) and not is_blacklisted(file):
+                    yield {"file_name": file, "file_content": self._read_file(file, branch=branch or self.active_branch)}
+        
+        return parse_code_files_for_db(file_content_generator())
 
     def get_available_tools(self):
         return [
@@ -578,7 +942,7 @@ class AlitaGitHubAPIWrapper(GitHubAPIWrapper):
             {
                 "ref": self.get_issue,
                 "name": "get_issue",
-                "mode":  "get_issue",
+                "mode": "get_issue",
                 "description": GET_ISSUE_PROMPT,
                 "args_schema": GetIssue,
             },
@@ -679,9 +1043,37 @@ class AlitaGitHubAPIWrapper(GitHubAPIWrapper):
                 "mode": "get_files_from_directory",
                 "description": GET_FILES_FROM_DIRECTORY_PROMPT,
                 "args_schema": DirectoryPath,
+            },
+            {
+                "ref": self.search_issues,
+                "name": "search_issues",
+                "mode": "search_issues",
+                "description": SEARCH_ISSUES_AND_PRS_PROMPT,
+                "args_schema": SearchIssues,
+            },
+            {
+                "ref": self.create_issue,
+                "name": "create_issue",
+                "mode": "create_issue",
+                "description": CREATE_ISSUE_PROMPT,
+                "args_schema": CreateIssue,
+            },
+            {
+                "ref": self.update_issue,
+                "name": "update_issue",
+                "mode": "update_issue",
+                "description": UPDATE_ISSUE_PROMPT,
+                "args_schema": UpdateIssue,
+            },
+            {
+                "ref": self.loader,
+                "name": "loader",
+                "mode": "loader",
+                "description": self.loader.__doc__,
+                "args_schema": LoaderSchema,
             }
         ]
-        
+
     def run(self, name: str, *args: Any, **kwargs: Any):
         for tool in self.get_available_tools():
             if tool["name"] == name:
