@@ -1,6 +1,7 @@
-from typing import List
+from typing import List, Literal
 
 from langchain_core.tools import BaseToolkit, BaseTool
+from pydantic import create_model, BaseModel, ConfigDict, Field
 
 from .api_wrapper import QtestApiWrapper
 from .tool import QtestAction
@@ -17,9 +18,22 @@ def get_tools(tool):
         qtest_api_token=tool['settings'].get('qtest_api_token', None),
     ).get_tools()
 
+
 class QtestToolkit(BaseToolkit):
     tools: List[BaseTool] = []
 
+    @staticmethod
+    def toolkit_config_schema() -> BaseModel:
+        selected_tools = {x['name']: x['args_schema'].schema() for x in QtestApiWrapper.model_construct().get_available_tools()}
+        return create_model(
+            name,
+            base_url=(str, Field(description="QTest base url")),
+            project_id=(int, Field(description="QTest project id")),
+            qtest_api_token=(str, Field(description="QTest API token", json_schema_extra={'secret': True})),
+            selected_tools=(List[Literal[tuple(selected_tools)]], Field(default=[], json_schema_extra={'args_schemas': selected_tools})),
+            __config__=ConfigDict(json_schema_extra={'metadata': {"label": "QTest", "icon_url": None}})
+        )
+
     @classmethod
     def get_toolkit(cls, selected_tools: list[str] | None = None, **kwargs):
         if selected_tools is None:
