@@ -1,5 +1,9 @@
+from typing import List, Literal
+
 from langchain_core.tools import BaseToolkit, BaseTool
 
+from pydantic import create_model, BaseModel, ConfigDict, Field
+
 from .api_wrapper import ReportPortalApiWrapper
 from ..base.tool import BaseAction
 
@@ -18,6 +22,18 @@ def get_tools(tool):
 class ReportPortalToolkit(BaseToolkit):
     tools: list[BaseTool] = []
 
+    @staticmethod
+    def toolkit_config_schema() -> BaseModel:
+        selected_tools = {x['name']: x['args_schema'].schema() for x in ReportPortalApiWrapper.model_construct().get_available_tools()}
+        return create_model(
+            name,
+            endpoint=(str, Field(description="Report Portal endpoint")),
+            project=(str, Field(description="Report Portal project")),
+            api_key=(str, Field(description="User API key", json_schema_extra={'secret': True})),
+            selected_tools=(List[Literal[tuple(selected_tools)]], Field(default=[], json_schema_extra={'args_schemas': selected_tools})),
+            __config__=ConfigDict(json_schema_extra={'metadata': {"label": "Report Portal", "icon_url": None}})
+        )
+
     @classmethod
     def get_toolkit(cls, selected_tools: list[str] | None = None, **kwargs):
         if selected_tools is None:
