@@ -1,4 +1,7 @@
+from typing import List, Literal
+
 from langchain_core.tools import BaseToolkit, BaseTool
+from pydantic import create_model, BaseModel, ConfigDict, Field
 
 from .api_wrapper import TestIOApiWrapper
 from ..base.tool import BaseAction
@@ -17,6 +20,17 @@ def get_tools(tool):
 class TestIOToolkit(BaseToolkit):
     tools: list[BaseTool] = []
 
+    @staticmethod
+    def toolkit_config_schema() -> BaseModel:
+        selected_tools = {x['name']: x['args_schema'].schema() for x in TestIOApiWrapper.model_construct().get_available_tools()}
+        return create_model(
+            name,
+            endpoint=(str, Field(description="TestIO endpoint")),
+            api_key=(str, Field(description="API key", json_schema_extra={'secret': True})),
+            selected_tools=(List[Literal[tuple(selected_tools)]], Field(default=[], json_schema_extra={'args_schemas': selected_tools})),
+            __config__=ConfigDict(json_schema_extra={'metadata': {"label": "TestIO", "icon_url": None}})
+        )
+
     @classmethod
     def get_toolkit(cls, selected_tools: list[str] | None = None, **kwargs):
         if selected_tools is None:
