@@ -1,7 +1,7 @@
-from typing import List
+from typing import List, Literal
 
-from langchain_community.agent_toolkits.base import BaseToolkit
-from langchain_core.tools import BaseTool
+from langchain_core.tools import BaseTool, BaseToolkit
+from pydantic import create_model, BaseModel, ConfigDict, Field
 
 from .api_wrapper import TestrailAPIWrapper
 from ..base.tool import BaseAction
@@ -20,6 +20,18 @@ def get_tools(tool):
 class TestrailToolkit(BaseToolkit):
     tools: List[BaseTool] = []
 
+    @staticmethod
+    def toolkit_config_schema() -> BaseModel:
+        selected_tools = {x['name']: x['args_schema'].schema() for x in TestrailAPIWrapper.model_construct().get_available_tools()}
+        return create_model(
+            name,
+            url=(str, Field(description="Testrail URL")),
+            email=(str, Field(description="User's email")),
+            password=(str, Field(description="User's password", json_schema_extra={'secret': True})),
+            selected_tools=(List[Literal[tuple(selected_tools)]], Field(default=[], json_schema_extra={'args_schemas': selected_tools})),
+            __config__=ConfigDict(json_schema_extra={'metadata': {"label": "Testrail", "icon_url": None}})
+        )
+
     @classmethod
     def get_toolkit(cls, selected_tools: list[str] | None = None, **kwargs):
         if selected_tools is None:
