@@ -1,7 +1,8 @@
-from typing import List
+from typing import List, Optional, Literal
 
 from langchain_community.agent_toolkits.base import BaseToolkit
 from langchain_core.tools import BaseTool
+from pydantic import create_model, BaseModel, Field
 
 from .api_wrapper import XrayApiWrapper
 from ..base.tool import BaseAction
@@ -23,6 +24,19 @@ def get_tools(tool):
 class XrayToolkit(BaseToolkit):
     tools: List[BaseTool] = []
 
+    @staticmethod
+    def toolkit_config_schema() -> BaseModel:
+        selected_tools = {x['name']: x['args_schema'].schema() for x in XrayApiWrapper.model_construct().get_available_tools()}
+        return create_model(
+            name,
+            base_url=(str, Field(description="Xray URL")),
+            client_id=(str, Field(description="Client ID")),
+            client_secret=(str, Field(description="Client secret", json_schema_extra={'secret': True})),
+            limit=(Optional[int], Field(description="Limit", default=100)),
+            selected_tools=(List[Literal[tuple(selected_tools)]], Field(default=[], json_schema_extra={'args_schemas': selected_tools})),
+            __config__={'json_schema_extra': {'metadata': {"label": "XRAY cloud", "icon_url": None}}}
+        )
+
     @classmethod
     def get_toolkit(cls, selected_tools: list[str] | None = None, **kwargs):
         if selected_tools is None:
