@@ -1,7 +1,8 @@
-from typing import List
+from typing import List, Literal, Optional
 
 from langchain_core.tools import BaseToolkit
 from langchain_core.tools import BaseTool
+from pydantic import create_model, BaseModel, Field
 
 from .yagmail_wrapper import YagmailWrapper, SMTP_SERVER
 from ..base.tool import BaseAction
@@ -19,7 +20,19 @@ def get_tools(tool):
 
 class AlitaYagmailToolkit(BaseToolkit):
     tools: List[BaseTool] = []
-    
+
+    @staticmethod
+    def toolkit_config_schema() -> BaseModel:
+        selected_tools = {x['name']: x['args_schema'].schema() for x in YagmailWrapper.model_construct().get_available_tools()}
+        return create_model(
+            name,
+            host=(Optional[str], Field(default=SMTP_SERVER, description="SMTP Host")),
+            username=(str, Field(description="Username")),
+            password=(str, Field(description="Password", json_schema_extra={'secret': True})),
+            selected_tools=(List[Literal[tuple(selected_tools)]], Field(default=[], json_schema_extra={'args_schemas': selected_tools})),
+            __config__={'json_schema_extra': {'metadata': {"label": "Yet Another Gmail", "icon_url": None}}}
+        )
+
     @classmethod
     def get_toolkit(cls, selected_tools: list[str] | None = None, **kwargs):
         if selected_tools is None:
